/*
	
					The Hacker's Diet

					 Diet Calculator
					 
*/

#include "Hackdiet.h"

static UInt16 activeField;				// ID of field being edited

static weightunit displayUnit;			// Weight unit for calculations
static char displayUnitName[30];		// Display unit name for pop-up trigger

static int calorieBalance;				// Calorie balance
static double startWeight,				// Starting weight
			  goalWeight,				// Goal weight
			  weightChange,				// Desired weight change
			  weightChangePerWeek;		// Weight change per week

static int weeksToGoal,					// Weeks to reach goal
		   monthsToGoal;				// Months to reach goal

static UInt32 startDate,					// Starting date
			 endDate;					// Ending date
			 
static RectangleType startDateRect, endDateRect;	// Start and end date field extents
	
#define Item(x) FrmGetObjectPtr(FrmGetActiveForm(), FrmGetObjectIndex(FrmGetActiveForm(), (x)))

#define edWeight(buf, w)	{ weightunit ow = prefs.displayUnit; \
							  prefs.displayUnit = displayUnit; \
							  editWeight(buf, w); \
							  prefs.displayUnit = ow; }
#define updWeight(fname, w)	edWeight(context, doubleToWeight(w)); \
							fldSetText(Item(fname), context)
							
#define weightToDouble(w)	((w) / 100.0)
#define doubleToWeight(d)	((weight) ((d) * 100))

#define Round(f)	((int) ((f) + (((f) < 0) ? -0.5 : 0.5)))
							
//	scanWeight  --  Scan a weight quantity from an ASCII string

static weight scanWeight(char *txt, weightunit wUnit)
{
	Char *decimal;
	weight w = 0;
	char weightField[24];

	assert(StrLen(txt) < (sizeof weightField));
	StrCopy(weightField, txt);
	txt = weightField;
	if (wUnit == wStone) {
		/*	When units are set to stones, accept entries of
			stones and pounds separated by a space.  An
			entry without a space is taken to be a round
			number of stones.  Pounds following stones may
			contain decimal places.  */
		decimal = StrChr(txt, ' ');
		if (decimal != NULL) {
			*decimal = 0;
		}
		NumIn(txt);
		w = StrAToI(txt) * 14 * 100;
		
		//	If a space was present, scan pounds and fraction
		
		if (decimal != NULL) {
			txt = decimal + 1;
			NumIn(txt);
			decimal = StrChr(txt, '.');
			
			if (decimal != NULL) {
				*decimal = 0;
			}
			w += StrAToI(txt) * 100;
			if (decimal != NULL) {
				decimal++;
				while (StrLen(decimal) > 2) {
					decimal[StrLen(decimal) - 1] = 0;
				}
				if (StrLen(decimal) < 2) {
					StrCat(decimal, "0");
				}
				w += StrAToI(decimal);
			}
		}
	} else {
		if (isDigit(txt[0])) {				
			NumIn(txt);
			decimal = StrChr(txt, '.');
			
			if (decimal != NULL) {
				*decimal = 0;
			}
			w = StrAToI(txt) * 100;
			if ((decimal != NULL) && 
				isDigit(decimal[1])) {
				decimal++;
				while (StrLen(decimal) > 2) {
					decimal[StrLen(decimal) - 1] = 0;
				}
				if (StrLen(decimal) < 2) {
					StrCat(decimal, "0");
				}
				w += StrAToI(decimal);
			}
		}
	}
	return w;
}

/*	signNum  --  Test for a sign preceding a number and return it.
				 The number is shifted in place to remove the sign
				 and any leading white space.  */
				 
static int signNum(char *s)
{
	int sgn = 1;
	char *cp = s, *op;
	
	while ((*cp != 0) && !isPrint(*cp)) {
		cp++;
	}
	if (!isDigit(*cp)) {
		if ((*cp == '+') || (*cp == '-')) {
			sgn = (*cp == '-') ? -1 : 1;
			op = s;
			cp++;
			while (*cp != 0) {
				*op++ = *cp++;
			}
			*op = 0;
		}
	}
	return sgn;
}

//	updateFields  --  Update fields in the form with their current values

static void updateFields(void)
{
	char context[80];
	DateTimeType dt;
	int energyBalance;

	if (PrefFlag(pfEnergyJoule)) {
		energyBalance = Round(calorieBalance / CaloriesPerJoule);
	} else {
		energyBalance = calorieBalance;
	}
#ifdef OS20BUG_WORKAROUND
	StrPrintF(context, "%c%d", energyBalance < 0 ? '-' : '+', abs(energyBalance));
#else
	StrPrintF(context, "%+d", energyBalance);
#endif
	fldSetText(Item(DietCalcCalorieBalanceField), context);
	updWeight(DietCalcInitialWeightField, startWeight);
	updWeight(DietCalcGoalWeightField, goalWeight);

	//	Total weight change
	edWeight(context + 1, doubleToWeight(abs(weightChange)));
	context[0] = weightChange >= 0 ? '+' : '-';
	fldSetText(Item(DietCalcDesiredWeightChangeField), context);

	//	Weight change per week
	edWeight(context + 1, doubleToWeight(abs(weightChangePerWeek)));
	context[0] = weightChangePerWeek >= 0 ? '+' : '-';
	fldSetText(Item(DietCalcChangePerWeekField), context);
	
	//	Weeks and months to reach goal
	if (weeksToGoal < 0) {
		/*	If we're going in the wrong direction, blank the
			otherwise negative weeks and months to go fields.  */
		fldSetText(Item(DietCalcWeeksToGoalField), "");
		fldSetText(Item(DietCalcMonthsToGoalField), "");
	} else {
		fldSetNumber(Item(DietCalcWeeksToGoalField), weeksToGoal);
		fldSetNumber(Item(DietCalcMonthsToGoalField), monthsToGoal);
	}

	//	Start date
	TimSecondsToDateTime(startDate, &dt);
	DateToAscii(dt.month, dt.day, dt.year,
				(DateFormatType) PrefGetPreference(prefLongDateFormat), context);
	fldSetText(Item(DietCalcStartDateField), context);

	//	End date
	if (endDate < startDate) {
		/*	If we're going in the wrong direction, show "Never" as
			the completion date.  */
		stringResource(NeverString, context, sizeof context);
	} else {
		TimSecondsToDateTime(endDate, &dt);
		DateToAscii(dt.month, dt.day, dt.year,
					(DateFormatType) PrefGetPreference(prefLongDateFormat), context);
	}
	fldSetText(Item(DietCalcEndDateField), context);
}
							
/*	recalculate  --  Recalculate all quantities from the daily
					 calorie balance, initial weight, and goal
					 weight, and starting date.  */
					 
static void recalculate(void)
{
	weightChange = goalWeight - startWeight;
	weightChangePerWeek = (calorieBalance * 7.0) / CaloriesPerWeightUnit(displayUnit);
	weeksToGoal = (int) (weightChange / weightChangePerWeek);
	monthsToGoal = (int) ((((weightChange / weightChangePerWeek) * 7.0) / 30.44) + 0.5);
	endDate = startDate + ((Int32) (weeksToGoal * 7.0 * 24.0 * 60.0 * 60.0));
	updateFields();
}

/*	updateField  --  Update numeric quantity corresponding to the
					 contents of a given field.  */
					 
static void updateField(void)
{
	FieldPtr f = Item(activeField);
	Char *txt;
	Char str[80];
	
	assert(f != NULL);
	txt = FldGetTextPtr(f);
	if (txt != NULL) {
		Int32 l;
		weight w;
		int sgn;
		
		switch (activeField) {
		
			/*	The calorie balance, start weight, and goal weight fields
				are the primary data from which the others are derived.  When
				one of these fields is modified, simply plug in the new value.  */
		
			case DietCalcCalorieBalanceField:
				l = StrAToI(txt);
				if (PrefFlag(pfEnergyJoule)) {
					calorieBalance = Round(l * CaloriesPerJoule);
				} else {
					calorieBalance = (int) l;
				}
				break;
				
			case DietCalcInitialWeightField:
				w = scanWeight(txt, displayUnit);
				if (w != 0) {
					startWeight = weightToDouble(w);
				}
				break;
				
			case DietCalcGoalWeightField:
				w = scanWeight(txt, displayUnit);
				if (w != 0) {
					goalWeight = weightToDouble(w);
				}
				break;
				
			/*	The user is permitted to edit the derived fields as well,
				each of which solves an equation in order to compute new
				values for the primary fields which will cause the derived
				value to be recomputed in a reasonable manner.
				
				Changing the desired change in weight modifies the
				goal weight to result in the specified delta.  */
				
			case DietCalcDesiredWeightChangeField:
				StrCopy(str, txt);
				sgn = signNum(str);
				w = scanWeight(str, displayUnit);
				if (w != 0) {
					goalWeight = startWeight + (weightToDouble(w) * sgn);
				}
				break;
				
			/*	When the weight change per week field is edited, it is
				used to adjust the calorie balance so as to accomplish
				the desired change.  */
				
			case DietCalcChangePerWeekField:
				StrCopy(str, txt);
				sgn = signNum(str);
				w = scanWeight(str, displayUnit);
				if (w != 0) {
					calorieBalance = (int) (((weightToDouble(w) * sgn) *
										CaloriesPerWeightUnit(displayUnit)) / 7);
				}
				break;
				
			/*	When the user changes the weeks to go field, calculate the
				calorie balance needed to accomplish the current weight change
				in the specified number of weeks.  */
				
			case DietCalcWeeksToGoalField:
				l = StrAToI(txt);
				if (l != 0) {
					calorieBalance = (int) (((weightChange / l) *
										CaloriesPerWeightUnit(displayUnit)) / 7);

				}
				break;
				
			/*  The months to go field triggers recomputation of the calorie
				balance needed to accomplish the current weight change in the
				specified number of months.  We take the number of days in a month
				as the mean for the Gregorian calendar.  */
				
			case DietCalcMonthsToGoalField:
				l = StrAToI(txt);
				if (l != 0) {
					calorieBalance = (int) (((weightChange / l) *
										CaloriesPerWeightUnit(displayUnit)) / 30.44);

				}
				break;
		}
	}
}

/*	doneEditing  --  Close out any field we're editing and update
					 accordingly.  */
					 
static void doneEditing(void)
{
	if (activeField != 0) {
		FieldPtr f = Item(activeField);
		
		assert(f != NULL);
		FldReleaseFocus(f);
		updateField();
		activeField = 0;
		recalculate();
	}
}

/*	dietCalcHandleEvent  --  Handle event while the diet calculator form
							 is displayed.  */							  

static Boolean dietCalcHandleEvent(EventPtr event)
{
	Boolean handled;
	FormPtr frm;
	char context[80];
	
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif

	handled = false;
	frm = FrmGetActiveForm();

	switch (event->eType) {
   		case ctlSelectEvent:  	// A control button was pressed and released.
			switch (event->data.ctlSelect.controlID) {
				case DietCalcDoneButton:		// Done button: Back to today's log
					monthLogShowToday();
					handled = true;
					break;
					
				case DietCalcChooseDisplayUnitsPopTrigger:	// Change weight display units
					{
						ListPtr units = Item(DietCalcDisplayUnitsList);
						short choice;
						weightunit oldDisplayUnit = displayUnit;
						
						LstSetSelection(units, displayUnit);
						choice = LstPopupList(units);
						if (choice >= 0) {
							displayUnit = (weightunit) choice;
							stringListItem(WeightUnitsStringList, displayUnit, displayUnitName, sizeof displayUnitName);
							CtlSetLabel(Item(DietCalcChooseDisplayUnitsPopTrigger), displayUnitName);
							
							/*	Now we must convert the starting and ending weights
								to the new units and recompute derivative quantities.  */
								
							startWeight = weightToDouble(cvtWeight(doubleToWeight(startWeight), oldDisplayUnit, displayUnit));
							/*	Change in displayed goal weight doesn't require a change
								to the preferences since the goal weight there is stored
								in the log unit.  */
							goalWeight = weightToDouble(cvtWeight(doubleToWeight(goalWeight), oldDisplayUnit, displayUnit));
							recalculate();
						}
					}
					handled = true;
					break;
			}
			break;
			
		case fldEnterEvent:
			if (activeField != 0) {
				recalculate();
			}
			activeField = event->data.fldEnter.fieldID;
			break;
			
		case penDownEvent:
		
			/*	If the user taps outside the field currently being
				edited, close out editing and recalculate.  */
				
			if (activeField != 0) {
				FieldPtr f = Item(activeField);
				RectangleType r;
				
				assert(f != NULL);
				FldGetBounds(f, &r);
				if (!RctPtInRectangle(event->screenX, event->screenY, &r)) {
					doneEditing();
				}
			}
			
			/*	Change to starting date:  The ending date will be adjusted to
				based on the new starting date and the existing duration
				estimate.  */
				
			if (RctPtInRectangle(event->screenX, event->screenY, &startDateRect)) {
				DateTimeType dt;
					
				TimSecondsToDateTime(startDate, &dt);
				stringResource(StartDateString, context, sizeof context);
				if (SelectDay(selectDayByDay, &dt.month, &dt.day, &dt.year, context)) {
					startDate = TimDateTimeToSeconds(&dt);
					recalculate();
				}
				handled = true;
			}
			
			/*	Change to ending date:  The calorie balance is adjusted so
				the diet ends on (approximately) the requested date.  */
				
			if (RctPtInRectangle(event->screenX, event->screenY, &endDateRect)) {
				DateTimeType dt;
					
				TimSecondsToDateTime(endDate, &dt);
				stringResource(EndDateString, context, sizeof context);
				if (SelectDay(selectDayByDay, &dt.month, &dt.day, &dt.year, context)) {
					endDate = TimDateTimeToSeconds(&dt);
					if (endDate > startDate) {		// Ignore if end date before start date
						calorieBalance = (int) ((weightChange / ((endDate - startDate) / (24.0 * 60 * 60))) *
										CaloriesPerWeightUnit(displayUnit));
					}
					recalculate();
				}
				handled = true;
			}
			break;
			
		case keyDownEvent:
			switch (event->data.keyDown.chr) {
			
				/*	If the user writes the line feed character, close
					out editing of any active field and recalculate.  */
			
				case linefeedChr:
					doneEditing();
					handled = true;
					break;
			}
			break;
  	
  		case frmOpenEvent:
  			/*	To simplify right alignment in localised versions,
  				we include separate balance labels for calories
  				and kilojoules, hand-aligned in the resource editor.
  				Here we blank the inapplicable one so they don't
  				step on one another.  Why not use CtlSetEnabled?
  				Because labels aren't controls--at least not enough
  				for the enabled flag to do us any good.  */
			if (PrefFlag(pfEnergyJoule)) {
#ifdef OLDWAY
				CtlSetLabel(Item(DietCalcBalanceCaloriesLabel), "");
#else
				FrmCopyLabel(FrmGetActiveForm(), DietCalcBalanceCaloriesLabel, "");
#endif
			} else {
#ifdef OLDWAY
				CtlSetLabel(Item(DietCalcBalanceKilojoulesLabel), "");
#else
				FrmCopyLabel(FrmGetActiveForm(), DietCalcBalanceKilojoulesLabel, "");
#endif
			}	
			FrmDrawForm(FrmGetActiveForm());
			stringListItem(WeightUnitsStringList, displayUnit, displayUnitName, sizeof displayUnitName);
			CtlSetLabel(Item(DietCalcChooseDisplayUnitsPopTrigger), displayUnitName);
			CtlSetValue(Item(DietCalcPlotPlanInChartCheckbox), PrefFlag(pfPlotPlan));
			recalculate();
			handled = true;
			break;
			
		case frmCloseEvent:
			//	Copy persistent state back to application preferences
			prefs.calcCalorieBalance = calorieBalance;
			prefs.calcStartWeight = cvtWeight(doubleToWeight(startWeight), displayUnit, prefs.weightUnit);
			prefs.calcGoalWeight = cvtWeight(doubleToWeight(goalWeight), displayUnit, prefs.weightUnit);
			prefs.calcStartDate = startDate;
			PrefFlagSet(pfPlotPlan, CtlGetValue(Item(DietCalcPlotPlanInChartCheckbox)));
			FrmEraseForm(FrmGetActiveForm());
			FrmDeleteForm(FrmGetActiveForm());
			handled = true;
			break;
	}
#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return handled;
}

/*	dietCalcLoad  --  Initialise diet calculator form and
					  display.  Called when ApplicationHandleEvent
					  received a frmLoadEvent for this 
					  form. */

void dietCalcLoad(FormPtr frm)
{
	int i;
	struct hdMonth *m;
	trend lastTrend = 0;
	weightunit lastUnit;
	UInt16 nMonths;

	FrmSetEventHandler(frm, dietCalcHandleEvent);
	displayUnit = prefs.displayUnit;
	activeField = 0;
	
	//	Save bounds of start and end date fields
	
	FrmGetObjectBounds(frm, FrmGetObjectIndex(frm, DietCalcStartDateField), &startDateRect);
	FrmGetObjectBounds(frm, FrmGetObjectIndex(frm, DietCalcEndDateField), &endDateRect);
	
	/*	Find the last weight entered in the log and use it as
		the starting weight in the form.  */
	
	nMonths = hdNumMonths();
	
	while ((lastTrend == 0) && (nMonths > 0)) {
		nMonths--;
		m = hdGetMonthUnpacked(nMonths);
		assert(m != NULL);
		for (i = m->days - 1; i >= 0; i--) {
			if (m->weightArray[i] != 0) {
				lastTrend = m->trendArray[i];
				lastUnit = (weightunit) m->weightUnit;
				break;
			}
		}
		hdDisposeMonthUnpacked(m);
	}
	
	/*	If lastTrend is still zero, we didn't find any weight data
		in the log.  This occurs when the user launches the diet
		calculator before entering any log data.  In this case, we
		simply have to plug in a default value for starting weight.  */
		
	if (lastTrend == 0) {
		lastUnit = displayUnit;
		lastTrend = WeightToTrend(doubleToWeight((lastUnit == wKilogram) ? 80 : 175));
	}
	
	//	Initialise source fields for the calculation
	
	calorieBalance = prefs.calcCalorieBalance;
	startWeight = weightToDouble(cvtWeight(prefs.calcStartWeight, prefs.weightUnit, displayUnit));
	if (startWeight == 0) {
		startWeight = (int) ((cvtTrend(lastTrend, lastUnit, displayUnit) / 1000000.0) + 0.5);
	}
	if (prefs.calcGoalWeight == 0) {
		goalWeight = startWeight - ((displayUnit == wKilogram) ? 5 : 10);
	} else {
		goalWeight = weightToDouble(cvtWeight(prefs.calcGoalWeight, prefs.weightUnit, displayUnit));
	}
	startDate = prefs.calcStartDate;
	if (startDate == 0) {
		startDate = TimGetSeconds();
	}
}