/*
	
					The Hacker's Diet

					   Preferences
					 
*/

#include "Hackdiet.h"

#define wPrefFlag(f)	((wprefs.preFlags & (1 << (f))) ? true : false)
#define wPrefFlagSet(f, b) wprefs.preFlags = (wprefs.preFlags & (~(1 << (f)))) | ((b) ? (1 << (f)) : 0)

//	System preferences

Char numberThousands, numberDecimal;	// Thousands and decimal separators for locale
Boolean hidePrivateData;				// Hide private records in database ?					

//	Form globals

struct hdPreferences prefs;			// Application preferences

static struct hdPreferences prefDefault = {	// Default preferences
	36,								// Months to keep in database on handheld
	0,								// Months actually kept on handheld at the moment
	wKilogram,						// Weight unit for monthly logs
	false,							// Consider monthly logs as private
	true,							// Protect data in logs from inadvertent changes
	wKilogram,						// Unit used to display weight
	0,								// Current rung in exercise ladder
	0,								// Boolean flag bits
	-500,							// Calorie balance
	0,								// Start weight (default is most recent trend value)
	0,								// Weight goal (default is none)
	0L,								// Start date (default is today)
	{
#define RGBColour(r, g, b)	{ 0, r, g, b }
#define GREYColour(v)		{ 0, v, v, v }
		RGBColour(CI_5, CI_0, CI_0),	// Monthly log plus variance
		RGBColour(CI_0, CI_4, CI_0),	// Monthly log minus variance
		GREYColour(0xDD),				// Chart background colour
		RGBColour(CI_5, CI_0, CI_0),	// 		Trend colour index
		RGBColour(CI_0, CI_4, CI_0),	// 		Float and sinker line colour index
		RGBColour(CI_0, CI_0, CI_5),	// 		Bob outline colour
		RGBColour(CI_5, CI_5, CI_0),	// 		Flagged day colour
		RGBColour(CI_2, CI_2, CI_5),	// 		Exercise rung colour
		RGBColour(CI_5, CI_3, CI_0)		// 		Diet plan colour

#undef RGBColour
#undef GREYColour
	}
};

/*	getPreferences  --  Load application preferences.  If they're not
						present, install our default application preferences
						in the database.  If the preferences block has changed
						in size since the preferences were stored, copy the
						additional fields from the default preferences into
						the end of the preferences buffer.  As long as you
						add new preferences at the end of the structure, the
						code will be upward compatible.  */
						
void getPreferences(void)
{
	UInt16 psize = sizeof prefs;
	SystemPreferencesType syspref;
	
	PrefGetPreferences(&syspref);
	hidePrivateData = syspref.hideSecretRecords;
	LocGetNumberSeparators((NumberFormatType) PrefGetPreference(prefNumberFormat), &numberThousands, &numberDecimal);
	if (PrefGetAppPreferences(CreatorID, 0, &prefs, &psize, true) == noPreferenceFound) {
		MemMove(&prefs, &prefDefault, sizeof prefs);
		PrefSetAppPreferences(CreatorID, 0, VersionNo, &prefs, sizeof prefs, true);
	} else {
		if (psize < (sizeof prefs)) {
			MemMove(((char *) &prefs) + psize, ((char *) &prefDefault) + psize, (sizeof prefs) - psize);
		}
	}
	
	/*	If this is colour machine, at this time we locate each of
		the colours in the application's colourScheme and set the
		index field of its RGBColorType structure to the slot
		containing that colour (or the closest match).  */

	if (colourDevice) {		
#define findRGB(colour)	PrefColourIndex(colour) = 0;  PrefColourIndex(colour) = WinRGBToIndex(&prefs.colourScheme.colour)
		findRGB(logPlusVarianceColour);
		findRGB(logMinusVarianceColour);
		findRGB(chartBackgroundColour);
		findRGB(chartTrendColour);
		findRGB(chartFlsinkColour);
		findRGB(chartBobColour);
		findRGB(chartFlagColour);
		findRGB(chartRungColour);
		findRGB(chartPlanColour);
#undef findRGB
	}
}

//	savePreferences  --  Save application preferences.

void savePreferences(void)
{
	PrefSetAppPreferences(CreatorID, 0, VersionNo, &prefs, sizeof prefs, true);
}

/*	preferencesHandleEvent  --  Handle event while the preferences form
								is displayed.  */							  

static Boolean preferencesHandleEvent(EventPtr event)
{
	Boolean handled;
	static struct hdPreferences wprefs;	// Working (non-committed) preferences
	FormPtr frm;
	static char utext_log_unit[30], utext_display_unit[30], utext_energy_unit[30];
	Char *cp;
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif

#define Item(x) FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, (x)))
	handled = false;
	frm = FrmGetActiveForm();

	switch (event->eType) {
   		case ctlSelectEvent:  // A control button was pressed and released.
			switch(event->data.ctlSelect.controlID) {
				case PreferencesPrefOKButton:		// OK button: Update preferences to current settings
					cp = FldGetTextPtr(Item(PreferencesMonthsToKeepField));
					wprefs.monthsKeep = StrAToI(cp);
					if (wprefs.monthsKeep < 1) {
						wprefs.monthsKeep = 1;
					}
					wprefs.privateLogs = CtlGetValue(Item(PreferencesPrivateLogsCheckbox));
					wprefs.protectEntries = CtlGetValue(Item(PreferencesProtectLogsCheckbox));
					MemMove(&prefs, &wprefs, sizeof prefs);	// Commit changes to current preferences
					/*	Any PrefFlagSet calls must be made after we copy wprefs back to prefs
						since they change prefs in place.  */
					PrefFlagSet(pfNoColour, !CtlGetValue(Item(PreferencesUseColourCheckbox)));
					
					/*	Now we need to adjust the use of colour if the user has changed
						the chromaticity preference on the fly.  */
						
					useColour = colourDevice && (!PrefFlag(pfNoColour));
					
					//	Wheee!!!  Note fall-through
					
				case PreferencesPrefCancelButton:
					{
						DateTimeType dt;

						/*	The first time the application is run after installation
							we come first to the Preferences form to invite the user
							to set the log and display weight unit.  After  the user
							completes this form, we now make sure the month's log
							exists, creating it in the chosen log units.  */
							
						TimSecondsToDateTime(TimGetSeconds(), &dt);
						hdEnsureMonthPresent(dt.year, dt.month - 1);
					}
					monthLogShowToday();
					handled = true;
					break;
					
				case PreferencesChooseWeightUnitsPopTrigger:
					{
						ListPtr units = Item(PreferencesWeightUnitsList);
						short choice;
						weightunit oldWeightUnit = wprefs.weightUnit;
						
						LstSetSelection(units, wprefs.weightUnit);
						choice = LstPopupList(units);
						if (choice >= 0) {
							wprefs.weightUnit = (weightunit) choice;
							stringListItem(WeightUnitsStringList, wprefs.weightUnit, utext_log_unit, sizeof utext_log_unit);
							CtlSetLabel(Item(PreferencesChooseWeightUnitsPopTrigger), utext_log_unit);
							wprefs.calcStartWeight = cvtWeight(wprefs.calcStartWeight, oldWeightUnit, wprefs.weightUnit);							
							wprefs.calcGoalWeight = cvtWeight(wprefs.calcGoalWeight, oldWeightUnit, wprefs.weightUnit);							
						}
					}
					handled = true;
					break;
					
				case PreferencesChooseDisplayUnitsPopTrigger:
					{
						ListPtr units = Item(PreferencesDisplayUnitsList);
						short choice;
						
						LstSetSelection(units, wprefs.displayUnit);
						choice = LstPopupList(units);
						if (choice >= 0) {
							wprefs.displayUnit = (weightunit) choice;
							stringListItem(WeightUnitsStringList, wprefs.displayUnit, utext_display_unit, sizeof utext_display_unit);
							CtlSetLabel(Item(PreferencesChooseDisplayUnitsPopTrigger), utext_display_unit);							
						}
					}
					handled = true;
					break;
					
				case PreferencesChooseEnergyUnitsPopTrigger:
					{
						ListPtr units = Item(PreferencesEnergyUnitsList);
						short choice;
						
						LstSetSelection(units, wPrefFlag(pfEnergyJoule) ? 1 : 0);
						choice = LstPopupList(units);
						if (choice >= 0) {
							wPrefFlagSet(pfEnergyJoule, choice == 1 ? true : false);
							stringListItem(EnergyUnitsStringList, wPrefFlag(pfEnergyJoule) ? 1 : 0, utext_energy_unit, sizeof utext_energy_unit);
							CtlSetLabel(Item(PreferencesChooseEnergyUnitsPopTrigger), utext_energy_unit);							
						}
					}
					handled = true;
					break;
					
				case PreferencesPrivateLogsCheckbox:
					{
						if (hidePrivateData && (!(event->data.ctlSelect.on))) {
							ControlPtr ct = event->data.ctlSelect.pControl;
							
							ct->attr.on = true;
							CtlDrawControl(ct);
							FrmAlert(PrivateLockedAlert);
						}
					}
					handled = true;
					break;
			}
			break;
  	
  		case frmOpenEvent:	
			MemMove(&wprefs, &prefs, sizeof wprefs);		// Set working preferences to current settings
			CtlSetValue(Item(PreferencesPrivateLogsCheckbox), wprefs.privateLogs);
			CtlSetValue(Item(PreferencesProtectLogsCheckbox), wprefs.protectEntries);
			CtlSetUsable(Item(PreferencesUseColourCheckbox), colourDevice);
			CtlSetValue(Item(PreferencesUseColourCheckbox), !PrefFlag(pfNoColour));
			stringListItem(WeightUnitsStringList, wprefs.weightUnit, utext_log_unit, sizeof utext_log_unit);
			CtlSetLabel(Item(PreferencesChooseWeightUnitsPopTrigger), utext_log_unit);
			stringListItem(WeightUnitsStringList, wprefs.displayUnit, utext_display_unit, sizeof utext_display_unit);
			CtlSetLabel(Item(PreferencesChooseDisplayUnitsPopTrigger), utext_display_unit);
			stringListItem(EnergyUnitsStringList, PrefFlag(pfEnergyJoule) ? 1 : 0, utext_energy_unit, sizeof utext_energy_unit);
			CtlSetLabel(Item(PreferencesChooseEnergyUnitsPopTrigger), utext_energy_unit);
			fldSetNumber(Item(PreferencesMonthsToKeepField), wprefs.monthsKeep);
			FrmDrawForm(FrmGetActiveForm());
			handled = true;
			break;
			
		case frmCloseEvent:
			FrmEraseForm(FrmGetActiveForm());
			FrmDeleteForm(FrmGetActiveForm());
			handled = true;
			break;
	}
#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return handled;
}

/*	preferencesLoad  --  Initialise preferences form and
						 display.  Called when ApplicationHandleEvent
						 received a frmLoadEvent for this 
						 form. */

void preferencesLoad(FormPtr f)
{
	FrmSetEventHandler(f, preferencesHandleEvent);
}