/*
	
					The Hacker's Diet

					 Year List Form
					 
		This file contains all the code for displaying and
		interacting with the Year List form, which displays
		a list of years present in the database.  Tapping on a
		year displays the calendar index to that year.
						
*/

#include "Hackdiet.h"			

//	Form globals

static char **yearListChoices = NULL;
static char *yearListText = NULL;

/*	yearListHandleEvent  --  Handle event while the year list form
							 is displayed.  */
							  

static Boolean yearListHandleEvent(EventPtr event)
{
	Boolean handled;
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif
	handled = false;


	switch (event->eType) {
			
		case ctlSelectEvent:
			switch(event->data.ctlSelect.controlID) {
				case YearListTodayButton:			// Today button: Show log for today
					{
						DateTimeType dt;
	
						TimSecondsToDateTime(TimGetSeconds(), &dt);
						goToToday = true;
						hdEnsureMonthPresent(dt.year, dt.month - 1);
						logGotoMonth = hdFindMonth(dt.year, dt.month - 1);
						FrmGotoForm(MonthlyLogForm);
						handled = true;
					}
					break;
					
				case YearListTrendButton:			// Trend button: Show trend analysis
					FrmGotoForm(TrendAnalysisForm);
					handled = true;
					break;
					
				case YearListMonthsButton:			// Months button: Show month list
					FrmGotoForm(MonthsForm);
					handled = true;
					break;
			}
			break;
  	
  		case frmOpenEvent:
			FrmDrawForm(FrmGetActiveForm());
			handled = true;
			break;
			
		case frmCloseEvent:
			if (yearListChoices != NULL) {
				MemPtrFree(yearListChoices);
				yearListChoices = NULL;
			}
			if (yearListText != NULL) {
				MemPtrFree(yearListText);
				yearListText = NULL;
			}
			FrmEraseForm(FrmGetActiveForm());
			FrmDeleteForm(FrmGetActiveForm());
			handled = true;
			break;

		case lstEnterEvent:
			LstEraseList(event->data.lstEnter.pList);
			LstSetSelection(event->data.lstEnter.pList, -1);
			LstDrawList(event->data.lstEnter.pList);
			/*	No, you *don't* want to set handled true here.  If you do,
				the list object's own tracking won't occur and scrolling,
				among other things, won't work.
			handled = true;
			*/
			break;
			
		case lstSelectEvent:			//	Since there's only one list, no need to check ID
			{
				yearGotoYear = StrAToI(yearListChoices[event->data.lstSelect.selection]);
				FrmGotoForm(YearViewForm);
				handled = true;
			}
			break;
			
		case keyDownEvent:
		
			/* You'd think that the list object, which dynamically
			   displays a scroll bar when more items are present than
			   fit on the screen, would also respond to the hardware
			   scroll buttons.  But it doesn't.  So the following code
			   handles key down events from the scroll buttons and
			   converts them into LstScrollList() calls to scroll the
			   list accordingly.  The logic is such that the scroll
			   buttons will be no-ops (will never call LstScrollList)
			   unless the list can actually be scrolled in that
			   direction. */
		
			switch (event->data.keyDown.chr) {
				case pageUpChr:
					{
						FormPtr f = FrmGetActiveForm();
						ListPtr lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, YearListYearListList));
						int lheight = LstGetVisibleItems(lp);
						int newtop = ((int) lp->topItem) - lheight;
						
						if (newtop < 0) {
							newtop = 0;
						}
						if (newtop != lp->topItem) {
							LstScrollList(lp, winUp, lp->topItem - newtop);
						}
						handled = true;
					}
					break;
					
				case pageDownChr:
					{
						FormPtr f = FrmGetActiveForm();
						ListPtr lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, YearListYearListList));
						int lheight = LstGetVisibleItems(lp);
						int newtop = ((int) lp->topItem) + lheight;
						
						if ((newtop + lheight) > lp->numItems) {
							newtop = lp->numItems - lheight;
						}
						if (newtop < 0) {
							newtop = 0;
						}
						if (newtop != lp->topItem) {
							LstScrollList(lp, winDown, newtop - lp->topItem);
						}
						handled = true;
					}
					break;
			}
			break;


#ifdef NEEDED			
		case menuEvent:		
			switch (event->data.menu.itemID) {
			
				case MonthsOptionsPreferences:
					FrmGotoForm(PreferencesForm);
					break;
				
				case MonthsOptionsAboutEatWatch:
					aboutDialogue();
//	   				FrmAlert(HackdietAlert);
					break;
					
				case MonthsOptionsTrendAnalysis:
					FrmGotoForm(TrendAnalysisForm);
					break;
				
					
#ifdef CSV
#ifdef CSVDEBUG
				case MonthsOptionsCSVDebug:
					FrmGotoForm(CSVDebugForm);
					break;
#endif
#endif
					
			}
			handled = true;
			break;
#endif
	}
#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return handled;
}

/*	yearListLoad  --   Initialise year list form and
					   display.  Called when ApplicationHandleEvent
					   received a frmLoadEvent for this year list
					   form. */

void yearListLoad(FormPtr f)
{
	int iter = -1, n = 0, currentYear, lYear = -1;
	UInt16 nmonths, year, month, thisYear = 0;
	ListPtr lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, YearListYearListList));
	char *cp;
#define ListItemMaxTextLength	MAX_MONTH_YYYY
		
	FrmSetEventHandler(f, yearListHandleEvent);

	currentYear = yearGotoYear;
	if (currentYear < 0) {
		DateTimeType dt;

		TimSecondsToDateTime(TimGetSeconds(), &dt);
		currentYear = dt.year;
	}
	yearGotoYear = -1;
	
	nmonths = hdNumMonths();
	if (nmonths > 0) {
		//	Way too big, but guaranteed to be enough (just enough, in the worst case)
		yearListChoices = MemPtrNew(sizeof(char *) * nmonths);
		if (yearListChoices == NULL) {
			return;
		}
		cp = yearListText = MemPtrNew((ListItemMaxTextLength + 1) * nmonths);
		if (yearListText == NULL) {
			MemPtrFree(yearListText);
			yearListText = NULL;
			return;
		}
		
		//	Allocation may have moved form and list--recover pointers
		f = FrmGetActiveForm();
		lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, YearListYearListList));
		
		while (true) {
			hdIterateMonths(&iter, &year, &month);
			if (year == 0 || iter == -1) {
				break;
			}
			assert(n < nmonths);
			if (((int) year) > lYear) {
				lYear = year;
				if (year == currentYear) {
					thisYear = n;
				}
				StrPrintF(cp, "%d", year);
				yearListChoices[n] = cp;
				cp += StrLen(cp) + 1;
				n++;
			}
		}
	}
	LstSetListChoices(lp, yearListChoices, n);

	/*	Select the current year and scroll so it's visible.
		The Palm OS SDK reference says LstSetSelection is all
		you need, but don't you believe it.  */
	
	LstSetSelection(lp, thisYear);
	LstMakeItemVisible(lp, thisYear);
}